<?php
/**
 * PercorreSEFIP.
 *
 * @package Logic
 *
 * @author  alessandrobartels
 */
class PercorreSEFIP{

    const CLT = 1;
    const SOCIO = 3;
    const ESTAGIARIO = 2;

    private $_parametros;
    private $_formulario;

    private $_retrocesso;

    private $_nrOrg;
    private $_nrOrgPadrao;
    private $_nrEstruturaTrab;
    private $_mesCompetencia;
    private $_nrTipoModalidadeCalculo;
   // private $_nrOcorrenciaCalculo;
    private $_tipoMovimento;
    private $_listaTipoRegistro;

    private $_dsInicio;
    private $_dsFim;
    private $_dsOrdem;
    private $_nrTpHierarquiaOrg;
    private $_tpInscHierarq;
    private $_nrTpHierarqTomador;
    private $_tpInscHierarqTomador;
    private $_idProcessaVinculo;
    private $_idGeraMeioMagnetico;
    private $_filePath;
    private $_nrEstruturaResp;
    private $_tpInscResp;
    private $_nrPessoaResp;
    private $_idRecolhimentoFGTS;
    private $_dtFGTS;
    private $_idRecolhimentoGPS;
    private $_dtGPS;
    private $_nrGrpIncide;

    private $_calculoFolha;
    private $_listaNrCalculoFolha;
    private $_gfipMagnetico;
    private $_tipoHierarquia;
    private $_hierarquiaOrg;
    private $_vinculom;
    private $_itemGrupo;

    private $_pessoa;
  //  private $_estrutura;
    private $_estruturacao;

    private $_sefip;
    private $_registro00;
    private $_registro90;
    private $_nrRecolhimento;
    private $_idModalidade;
    private $_competencia13;


    /**
     * Construtor.
     *
     * @param string     $tpProcessamento Tipo de processamento (MENSAL / COMPETENCIA 13).
     * @param integer    $nrFormulario    Numero do formulario.
     * @param Parametros $parametro       Instancia de Parametros.
     *
     * @author alessandrobartels
     */
    public function __construct($tpProcessamento, $nrFormulario, Parametros $parametro){

        $this->_parametros = $parametro;
        $this->_nrOrg = $this->_parametros->getNrOrgTrabalho();
        $this->_nrOrgPadrao = $this->_parametros->getNrOrgPadrao();
        $this->_nrEstruturaTrab = \FPA::getEstruturaTrabalho();
        $this->_mesCompetencia = new TkDateTime($this->_parametros->getMesCompetencia());
        $this->_nrTipoModalidadeCalculo = $this->_parametros->getTipoModalidadeCalculo();
        //$this->_nrOcorrenciaCalculo = $this->_parametros->getNrOcorrenciaCalculo();

        // $this->_tipoProcessamento = $tpProcessamento;.
        $this->_tipoMovimento = FpatipomovimentEntity::TP_MOV_MENSAL;
        // $this->_nrRecolhimento = $nrRecolhimento;.
        // $this->_idModalidade = $idModalidade;.
        $this->_itemGrupo = new FpaGrpitemEntity();

        $this->_vinculom = new GpevinculomEntity();
        $this->_tipoHierarquia = new TipohierarqorgEntity();
        $this->_hierarquiaOrg = new HierarquiaorgEntity();
        $this->_calculoFolha = new FpacalculofolhaEntity();
        $this->_gfipMagnetico = new FpagfipmagnetEntity();
        $this->_registro00 = new Registro00SEFIP();
        $this->_registro90 = new Registro90SEFIP();

        $this->_pessoa = new GpepessoaEntity(null, $this->_nrOrg, $this->_mesCompetencia);
       // $this->_estrutura = new EstruturamEntity(null, $this->_nrOrg, $this->_mesCompetencia);
        $this->_estruturacao = new Estruturacao($this->_nrOrg, $this->_nrOrgPadrao, $this->_mesCompetencia);

        $this->_inicializaVariaveis($nrFormulario);

        $tpProcessamento = strtoupper($tpProcessamento);
        switch ($tpProcessamento){
        case Parametros::PROC_COMPETENCIA_MENSAL :
             $this->_retrocesso = false;
             $this->_competencia13 = false;
             break;
        case Parametros::PROC_COMPETENCIA_13 :
             $this->_retrocesso = false;
             $this->_competencia13 = true;
             break;
        case Parametros::PROC_RETROCESSO :
            $this->_retrocesso = true;
            $this->_competencia13 = false;
            break;
        default:
            tk_message('Tipo de processo '.$tpProcessamento.' invalido', 'warning');
        }

        $this->_sefip  = new SEFIP(
            $this->_nrOrg, $this->_nrOrgPadrao, $this->_mesCompetencia, $this->_gfipMagnetico, $this->_formulario,
            $this->_competencia13, $this->_itemGrupo, $this->_listaNrCalculoFolha
        );
    }


    public function _inicializaVariaveis($nrFormulario) {

        $this->_buscaDadosFormulario($nrFormulario);
        $this->_itemGrupo->carregaItensGrupoPorGrupoIncidencia($this->_nrOrg, $this->_nrOrgPadrao, $this->_nrGrpIncide);
        // Cria lista de nr_calculo_folha.
        $this->_listaNrCalculoFolha = $this->_calculoFolha->carregaListaNrCalculoFolhaTodasOcorrencias(
            $this->_nrOrg, $this->_tipoMovimento, $this->_mesCompetencia, $this->_nrTipoModalidadeCalculo
        );
    }

    /**
     * Percorre os vinculos para geracao dos registros.
     *
     * @throws Exception Propaga excecao.
     *
     * @return void
     *
     * @author alessandrobartels
     */
    public function percorre(){
        if ($this->_retrocesso === false) {            
            try{
                // verifica se existe magnetico, e pede para retroceder - leonardocarvalho 22/08/2014
                $registros = $this->_gfipMagnetico->retornaRegistros($this->_mesCompetencia, $this->_nrOrg);
                if (count($registros) == 0) {
                    $this->_geraSEFIP();
                    tk_message('Arquivo gerado', 'success');
                } else {
                    tk_warning('Já existe magnético do Sefip para o mês de competência, favor retroceder.');
                }
            }catch (Exception $e) {
                //throw new Exception($e);
                tk_warning($e->getMessage());
            }
        } else {
            try {
                $this->_sefip->retroceder();
                tk_message('Retrocedido', 'success');
            } catch (Exception $e) {
                //throw new Exception($e);
                tk_warning($e->getMessage());
            }
        }
    }


    /**
     * Gerar os registros do arquivo SEFIP.RE.
     *
     * @throws Exception Propaga Excecao.
     *
     * @return void
     *
     * @author alessandrobartels
     *         leonardocarvalho - 28/05/2014 - Condição para ignorar somente o tipo de vinculo Estagiário, pois tipo de vinculo SOCIO não estava entrando no SEFIP.
     */
    private function _geraSEFIP(){

        // Percorre os vinculos gerando os registros do SEFIP.
        if ($this->_idProcessaVinculo === true) {
            // Inicializa Lista do Magnetico. Carrega registros de quebra de empresa e tomador (10, 12, 20, 21).
            $adicionou_itens = $this->_gfipMagnetico->inicializaListaRegistros(
                $this->_mesCompetencia, RegistroSEFIP::LISTA_TIPO_REG_EMP, $this->_nrOrg
            );

            // Retorna todos os vinculos.
            $vinculos = $this->_vinculom->retornaVinculosAtivosMesCompetencia($this->_nrOrg, $this->_mesCompetencia,
                $this->_dsInicio, $this->_dsFim, $this->_dsOrdem, true, $this->_nrEstruturaTrab);

            // Percorre os vinculos gerando sefip.
            /** @var $vinculo GpevinculomEntity */
            foreach ($vinculos as $vinculo) {
                // So tem SEFIP o tipo de vinculo CLT
                if ($vinculo->getNrtipovinculom() == self::ESTAGIARIO) {
                    continue;
                }
                // Somente avalia os funcionarios com vinculo empregaticio.
                if ($vinculo->getNrvinculoempreg() !== false) {
//                    $this->_sefip->geraSEFIP($vinculo, $this->_tpInscHierarq, $this->_tpInscHierarqTomador);
                    $this->_sefip->geraSEFIP($vinculo, $this->_nrTpHierarquiaOrg, $this->_nrTpHierarqTomador, $this->_tpInscHierarq, $this->_tpInscHierarqTomador);
                } else {
                        throw new \Exception('Vinculo empregaticio nao encontrado para o vinculo '.$vinculo->getNrvinculom());
                }
                \Setforms2\Runtime\Connection::getInstance()->clear();
            }

            // Modifica valores nas strings ja montadas.
            $this->_sefip->modificaSalarioMaternidadeFamilia();
                // criar metodo no SEFIP para fazer a modificacao

            // So persiste se a lista nao estiver vazia.
            $qtdRegistros = $this->_gfipMagnetico->countItems();
//            $qtdRegistros = count($this->_gfipMagnetico->getItems());
            if ($qtdRegistros > 0) {
                $this->_gfipMagnetico->persist();
            }
        }

        // Gera o magnetico.
        if ($this->_idGeraMeioMagnetico === true) {
            $this->_geraMagnetico();
        }
    }


    /**
     * Enter description here ...
     *
     * @param string          $tpInscricao Tipo de inscricao responsavel pela geracao do arquivo.
     * @param string          $nrInscricao Numero de inscricao da empresa responsavel pela geracao do arquivo.
     *
     * @return void
     *
     * @author alessandrobartels
     */
    private function _geraMagnetico($tpInscricao = null, $nrInscricao = null){

			//Chama m�todo para gerar o arquivo, passando os par�metros: nome da pasta, nome do arquivo e extens�o arquivo
        $nomePasta = "SEFIP";
        $nomeArquivo = "SEFIP.RE";
        $extensao = ".RE";
        $arquivo = \FPA::geraArquivo($nomePasta, $nomeArquivo, $extensao);

        $this->_preparaRegistro00();

        /* Informacoes sobre o responsavel do arquivo. */
        $this->_registro00->geraRegistro00();

        // Armazenar o registro 00 com quebra de linha.
        $arquivo->add($this->_registro00->getLinhaRegistro().RegistroSEFIP::QUEBRA_LINHA_CR_LF);

        // Buscar da base os resgistros daquela competencia, ordenados por  inscricao  e tipo de registro.
        $resultado = $this->_gfipMagnetico->retornaRegistros($this->_mesCompetencia, $this->_nrOrg);

        // Loop no resultado e armazenar os registros com quebra de linha.
        foreach ($resultado as $item) {
            $arquivo->add($item->getDslinharegistro().RegistroSEFIP::QUEBRA_LINHA_CR_LF);
        }

        // Gerar Registro90.
        $this->_registro90->geraRegistro90();

        //adiciona
        $arquivo->add($this->_registro90->getLinhaRegistro());
        $arquivo->save(false, null, 0777);
        $arquivo->download(substr($_SERVER['REQUEST_URI'], 0, strlen($_SERVER['REQUEST_URI']) - 12)."/".\FPA::getNrorgTrabalho()."/".$nomePasta."/".$nomeArquivo);

    }


    private function _preparaRegistro00(){

        //@todo validar caso nao exista estrutura e pessoa parametrizada.
//        $estrutura = $this->_estrutura->retornaEstrutura($this->_nrEstruturaResp);
        $estrutura = $this->_estruturacao->retornaEstrutura($this->_nrEstruturaResp);
        if (is_object($estrutura) === true){
            $estrutura->endereco = $this->_estruturacao->determinaEnderecoEstrutura($estrutura);
            $pessoa = $this->_pessoa->retornaPessoaAtual($this->_nrPessoaResp);

            if ($this->_tpInscResp === RegistroSEFIP::CNPJ) {
                $this->_registro00->setTpInscricao(RegistroSEFIP::CNPJ);
                $this->_registro00->setNrInscricao($estrutura->historico->getCdcnpjestrut());
            } elseif ($this->_tpInscResp === RegistroSEFIP::CEI) {
                $this->_registro00->setTpInscricao(RegistroSEFIP::CEI);
                $this->_registro00->setNrInscricao($estrutura->historico->getCdceiestrut());
            }

            $this->_registro00->setPessoaResp($pessoa);
            $this->_registro00->setEstruturaResp($estrutura);
            $this->_registro00->setCompetencia($this->_mesCompetencia);
            $this->_registro00->setCodigoRecolhimento($this->_nrRecolhimento);
            $this->_registro00->setModalidade($this->_idModalidade);
            $this->_registro00->setIdRecolhimentoFGTS($this->_idRecolhimentoFGTS);
            $this->_registro00->setDtFGTS($this->_dtFGTS);
            $this->_registro00->setIdRecolhimentoGPS($this->_idRecolhimentoGPS);
            $this->_registro00->setDtGPS($this->_dtGPS);
            $this->_registro00->setCompetencia13($this->_competencia13);
        }
    }

    /**
     * Busca dados do formulario.
     *
     * @param integer $nrFormulario Numero do formulario.
     *
     * @throws Exception
     * @return void
     *
     * @author alessandrobartels
     */
    private function _buscaDadosFormulario($nrFormulario){
        $formulario = new FpaformgfipEntity();
        $this->_formulario = $formulario->retornaDadosFormularioGFIP($this->_nrOrg, $nrFormulario);
        if (is_object($this->_formulario) === true) {
            $this->_nrRecolhimento = $this->_formulario->getNrrecolhimento();
            $this->_idModalidade = $this->_formulario->getIdmodalidade();
            $this->_dsInicio = $this->_formulario->getDsinicio();
            $this->_dsFim = $this->_formulario->getDsfim();
            $this->_dsOrdem = $this->_formulario->getDsordem();
            $this->_nrTpHierarquiaOrg = $this->_formulario->getNrhierarqorg();
            $this->_tpInscHierarq = $this->_formulario->getIdtpinschierarq();
            $this->_nrTpHierarqTomador = $this->_formulario->getNrhierartomador();
            $this->_tpInscHierarqTomador = $this->_formulario->getIdtpinschiertom();
            $this->_idProcessaVinculo = $this->_formulario->getIdprocvinculo();
            $this->_idGeraMeioMagnetico = $this->_formulario->getIdgerameiomag();
            $this->_filePath = $this->_formulario->getDscaminhoarquivo();
            $this->_nrEstruturaResp = $this->_formulario->getNrestrutrespons();
            $this->_tpInscResp = $this->_formulario->getIdtpinscrespons();
            $this->_nrPessoaResp = $this->_formulario->getNrpessoarespons();
            $this->_idRecolhimentoFGTS = $this->_formulario->getIdrecolhfgts();
            $this->_dtFGTS = $this->_formulario->getDtrecolhfgts();
            $this->_idRecolhimentoGPS = $this->_formulario->getIdrecolhgps();
            $this->_dtGPS = $this->_formulario->getDtrecolhgps();
            $this->_nrGrpIncide = $this->_formulario->getNrgrpincide();
        } else {
            // LEONARDOCARVALHO 26/09/2014
            throw new \Exception("Formulário não encontrado, contate o administrador.");
        }
    }


}